/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Mon Jun  3 15:56:02 CEST 2002
    copyright            : (C) 2002 by Andre Stein
    email                : andre_stein@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream.h>
#include <SDL.h>
#include "sglwindow.h"

#define X_SIZE 640
#define Y_SIZE 480

//global window object..
SGLWindow window;
SConfiguration configFile;

//this functions handles all SDL
//events every loop
bool handleEvents()
{
	SDL_Event event;
	
 	//Next event
	while (SDL_PollEvent(&event))
	{
		//Check event
		switch(event.type)
		{
		case SDL_KEYDOWN:
                         //Check key
                         window.keyDown(event.key.keysym.sym);
			 break;
		case  SDL_QUIT:
			return false;
			break;
		default:
			break;
		}
	}

	return true;
}

#ifdef WIN32
int WINAPI WinMain(HINSTANCE	hInstance,			
                   HINSTANCE	hPrevInstance,		
                   LPSTR		lpCmdLine,	
                   int			nCmdShow)
#else
//the great starting point ;-)
int main(int argc, char *argv[])
#endif
{           
        unsigned int start,
                     end;


        //Under Linux, we have to create the profile dir
#ifndef WIN32
        system(CONFIG_DIR_CMD);
#endif
                     
        //Load config file
        configFile.load(CONFIG_FILENAME);
        
	//***********************
	//SDL init
	//***********************

	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0)
	{
		cout << "Couldn't init SDL" << endl;
		return -1;
	}

	//************************
	//Screen Init
	//************************

	SDL_Surface* screen;

	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );

        //set video mode and depending on value in config file,
        //either fullscreen or not
	screen = SDL_SetVideoMode(X_SIZE,Y_SIZE,0,SDL_OPENGL |
                        (configFile.isFullscreen() ? SDL_FULLSCREEN : 0)
                        );
	
	//***********************
	//DEBUG
	//***********************
	
	char buffer[512];
	SDL_VideoDriverName(buffer, 512);
	
	cout << "SDL Video Driver-name: " << buffer << endl;
	cout << "OpenGL Infos" << endl
 	     << "-------------------------------" << endl;
	cout << " Vendor: "     << glGetString(GL_VENDOR) << endl
	     << " Extensions: " << glGetString(GL_EXTENSIONS) << endl;
	cout << "-------------------------------"
 	     << endl;

	
        if (screen == NULL)
	{
		cout << "Couldn't go fullscreen" << endl;
	  	cout << SDL_GetError() << endl;
	}
        
	//Keyboard
	SDL_EnableKeyRepeat(100,150);

        //Init Window
	window.init(&configFile, screen);
	window.resize(X_SIZE,Y_SIZE);

        //don't show cursor
        SDL_ShowCursor(false);
 
	while (true)
	{
		if (! handleEvents() )
		{
			break;
		}

               /* TIMER */

                start = SDL_GetTicks();

                do
                {
                        end = SDL_GetTicks();
                }
                while ( start == end );

                /* DRAW WINDOW */
		
		window.update(end-start);
		
		//Swap  buffers
		SDL_GL_SwapBuffers();
	}

        SDL_ShowCursor(true);

        //save config file
        configFile.save(CONFIG_FILENAME);
        
	//Shutdown SDL
	SDL_Quit();

        

	return 0;
}
