/***************************************************************************
                          sbrick.cpp  -  description
                             -------------------
    begin                : Sun Dec 22 2002
    copyright            : (C) 2002 by Andre Stein
    email                : andre_stein@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sbrick.h"
#include <math.h>

const float PI_OVER_2 = 3.14159f*0.5f;
const float SIN_ANGLE = sin(PI_OVER_2);
const float COS_ANGLE = cos(PI_OVER_2);

SBrick::SBrick()
{
        rotation = 0.0f;
}

SBrick::~SBrick()
{
}
/** No descriptions */
void SBrick::create(SBrickType brickType, SBMPTexture* textures)
{
        brick = brickType;
        rotation = 0.0f;
/** BRICKS:
 1: **  2: *  3: *  4: *  5: ** 6:  * 7: **
    *      *     **    **    **    **     *
    *      *      *    *           *      *
           *                        
*/

        switch (brick)
        {
        case BRICK1:
                boxes[0].setPosition(0.0f,0.0f);
                boxes[1].setPosition(1.0f,0.0f);
                boxes[2].setPosition(0.0f,-1.0f);
                boxes[3].setPosition(0.0f,-2.0f);
                boxes[0].init(&textures[BRICK1]);
                boxes[1].init(&textures[BRICK1]);
                boxes[2].init(&textures[BRICK1]);
                boxes[3].init(&textures[BRICK1]);
                break;
        case BRICK2:
                boxes[0].setPosition(0.0f,0.0f);
                boxes[1].setPosition(0.0f,-1.0f);
                boxes[2].setPosition(0.0f,-2.0f);
                boxes[3].setPosition(0.0f,-3.0f);
                boxes[0].init(&textures[BRICK2]);
                boxes[1].init(&textures[BRICK2]);
                boxes[2].init(&textures[BRICK2]);
                boxes[3].init(&textures[BRICK2]);
                break;
        case BRICK3:
                boxes[0].setPosition(0.0f,0.0f);
                boxes[1].setPosition(0.0f,-1.0f);
                boxes[2].setPosition(1.0f,-1.0f);
                boxes[3].setPosition(1.0f,-2.0f);
                boxes[0].init(&textures[BRICK3]);
                boxes[1].init(&textures[BRICK3]);
                boxes[2].init(&textures[BRICK3]);
                boxes[3].init(&textures[BRICK3]);
                break;
        case BRICK4:
                boxes[0].setPosition(0.0f,0.0f);
                boxes[1].setPosition(0.0f,-1.0f);
                boxes[2].setPosition(1.0f,-1.0f);
                boxes[3].setPosition(0.0f,-2.0f);
                boxes[0].init(&textures[BRICK4]);
                boxes[1].init(&textures[BRICK4]);
                boxes[2].init(&textures[BRICK4]);
                boxes[3].init(&textures[BRICK4]);
                break;
        case BRICK5:
                boxes[0].setPosition(0.0f,0.0f);
                boxes[1].setPosition(1.0f,0.0f);
                boxes[2].setPosition(0.0f,-1.0f);
                boxes[3].setPosition(1.0f,-1.0f);
                boxes[0].init(&textures[BRICK5]);
                boxes[1].init(&textures[BRICK5]);
                boxes[2].init(&textures[BRICK5]);
                boxes[3].init(&textures[BRICK5]);
                break;
        case BRICK6:
                boxes[0].setPosition(1.0f,0.0f);
                boxes[1].setPosition(1.0f,-1.0f);
                boxes[2].setPosition(0.0f,-1.0f);
                boxes[3].setPosition(0.0f,-2.0f);
                boxes[0].init(&textures[BRICK6]);
                boxes[1].init(&textures[BRICK6]);
                boxes[2].init(&textures[BRICK6]);
                boxes[3].init(&textures[BRICK6]);
                break;
        case BRICK7:
                boxes[0].setPosition(0.0f,0.0f);
                boxes[1].setPosition(1.0f,0.0f);
                boxes[2].setPosition(1.0f,-1.0f);
                boxes[3].setPosition(1.0f,-2.0f);
                boxes[0].init(&textures[BRICK7]);
                boxes[1].init(&textures[BRICK7]);
                boxes[2].init(&textures[BRICK7]);
                boxes[3].init(&textures[BRICK7]);
                break;
        }
}

/** No descriptions */
void SBrick::moveBy(float x, float y)
{
        boxes[0].moveBy(x,y);
        boxes[1].moveBy(x,y);
        boxes[2].moveBy(x,y);
        boxes[3].moveBy(x,y);
}
/** No descriptions */
void SBrick::draw()
{
        boxes[0].draw();
        boxes[1].draw();
        boxes[2].draw();
        boxes[3].draw();
}

/** No descriptions */
void SBrick::moveLeft()
{
        if (! (boxes[0].touchesLeft() || boxes[1].touchesLeft() ||
                boxes[2].touchesLeft() || boxes[3].touchesLeft()
                || touchesBoxes(LEFT) ) )
        {
                boxes[0].moveBy(-1.0f,0.0f);
                boxes[1].moveBy(-1.0f,0.0f);
                boxes[2].moveBy(-1.0f,0.0f);
                boxes[3].moveBy(-1.0f,0.0f);                
        }
}

/** No descriptions */
void SBrick::moveRight()
{
        if (! (boxes[0].touchesRight() || boxes[1].touchesRight() ||
                boxes[2].touchesRight() || boxes[3].touchesRight() ||
                touchesBoxes(RIGHT) ) )
        {
                boxes[0].moveBy(1.0f,0.0f);
                boxes[1].moveBy(1.0f,0.0f);
                boxes[2].moveBy(1.0f,0.0f);
                boxes[3].moveBy(1.0f,0.0f);
        }
}

/** No descriptions */
void SBrick::moveDown()
{
        if (! (boxes[0].touchesBottom() || boxes[1].touchesBottom() ||
                boxes[2].touchesBottom() || boxes[3].touchesBottom() ||
                touchesBoxes(BOTTOM) ) )
        {
                boxes[0].moveBy(0.0f,-1.0f);
                boxes[1].moveBy(0.0f,-1.0f);
                boxes[2].moveBy(0.0f,-1.0f);
                boxes[3].moveBy(0.0f,-1.0f);
        }
}

/** No descriptions */
bool SBrick::touchesBottom()
{
        return (boxes[0].touchesBottom() || boxes[1].touchesBottom() ||
                boxes[2].touchesBottom() || boxes[3].touchesBottom());
}

/** No descriptions */
bool SBrick::touchesBoxes(SDirection pos)
{
        return (boxes[0].touchesBoxes(pos) || boxes[1].touchesBoxes(pos) ||
                boxes[2].touchesBoxes(pos) || boxes[3].touchesBoxes(pos));
}
/** No descriptions */
void SBrick::changeForm()
{
/** BRICKS:
 1: **  2: *  3: *  4: *  5: ** 6:  * 7: **
    *      *     **    **    **    **     *
    *      *      *    *           *      *
           *
*/
         //the quadric block won't change anyway
        if ( brick == BRICK5 )
                return;

        rotation += 90.0f;
        if ( rotation == 360.0f)
                rotation = 0.0f;

        switch (brick)
        {
        /*  **  90: *  180:  *  270: 
            *       ***      *       ***
            *               **         *
        */
        case BRICK1:
                if (rotation == 90.0f)
                {
                        boxes[0].moveBy(-1.0f,-1.0f);
                        boxes[1].moveBy(-2.0f,0.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(1.0f,1.0f);
                }
                else if (rotation == 180.0f)
                {
                        boxes[0].moveBy(1.0f,-1.0f);
                        boxes[1].moveBy(0.0f,-2.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(-1.0f,1.0f);                
                }
                else if (rotation == 270.0f)
                {
                        boxes[0].moveBy(1.0f,1.0f);
                        boxes[1].moveBy(2.0f,0.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(-1.0f,-1.0f);
                }
                else if (rotation == 0.0f)
                {
                        boxes[0].moveBy(-1.0f,1.0f);
                        boxes[1].moveBy(0.0f,2.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(1.0f,-1.0f);
                }
                break;
        /*  *    90:     180:  * 270:
            *      ****        *     ****
            *                  *
            *                  *
        */
        case BRICK2:
                if (rotation == 90.0f || rotation == 270.0f)
                {
                        boxes[0].moveBy(1.0f,-1.0f);
                        boxes[1].moveBy(0.0f,0.0f);
                        boxes[2].moveBy(-1.0f,1.0f);
                        boxes[3].moveBy(-2.0f,2.0f);
                }
                else if (rotation == 180.0f || rotation == 0.0f)
                {
                        boxes[0].moveBy(-1.0f,1.0f);
                        boxes[1].moveBy(0.0f,0.0f);
                        boxes[2].moveBy(1.0f,-1.0f);
                        boxes[3].moveBy(2.0f,-2.0f);
                }
                
                break;
   /** BRICKS:
              0: *  90  **  180: -> 0 270: -> 90
                 **    **       
                  *                     
*/
        case BRICK3:
                if (rotation == 90.0f || rotation == 270.0f)
                {
                        boxes[0].moveBy(2.0f,0.0f);
                        boxes[1].moveBy(1.0f,1.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(-1.0f,1.0f);
                }
                else if (rotation == 180.0f || rotation == 0.0f)
                {
                        boxes[0].moveBy(-2.0f,0.0f);
                        boxes[1].moveBy(-1.0f,-1.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(1.0f,-1.0f);
                }
                break;
        /*
          *   90:  *   180:  *   270: ***
          **      ***       **         *
          *                  *
        */
        case BRICK4:
                if (rotation == 90.0f)
                {
                        boxes[0].moveBy(-1.0f,-1.0f);
                        boxes[1].moveBy(0.0f,0.0f);
                        boxes[2].moveBy(-1.0f,1.0f);
                        boxes[3].moveBy(1.0f,1.0f);
                }
                else if (rotation == 180.0f)
                {
                        boxes[0].moveBy(1.0f,-1.0f);
                        boxes[1].moveBy(0.0f,0.0f);
                        boxes[2].moveBy(-1.0f,-1.0f);
                        boxes[3].moveBy(-1.0f,1.0f);
                }
                else if (rotation == 270.0f)
                {
                        boxes[0].moveBy(1.0f,1.0f);
                        boxes[1].moveBy(0.0f,0.0f);
                        boxes[2].moveBy(1.0f,-1.0f);
                        boxes[3].moveBy(-1.0f,-1.0f);
                }
                else if (rotation == 0.0f)
                {
                        boxes[0].moveBy(-1.0f,1.0f);
                        boxes[1].moveBy(0.0f,0.0f);
                        boxes[2].moveBy(1.0f,1.0f);
                        boxes[3].moveBy(1.0f,-1.0f);
                }
                break;
        case BRICK5:
                break;
        /** BRICKS:
              0:  * 90 **   180: -> 0 270: -> 90
                 **     **
                 *
        */
        case BRICK6:
                if (rotation == 90.0f || rotation == 270.0f)
                {
                        boxes[0].moveBy(-2.0f,0.0f);
                        boxes[1].moveBy(-1.0f,1.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(1.0f,1.0f);
                }
                else if (rotation == 180.0f || rotation == 0.0f)
                {
                        boxes[0].moveBy(2.0f,0.0f);
                        boxes[1].moveBy(1.0f,-1.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(-1.0f,-1.0f);
                }
                break;
        /*
              **   90:       180:  *   270:   *
               *        ***        *        ***
               *        *          **
        */
        case BRICK7:
                if (rotation == 90.0f)
                {
                        boxes[0].moveBy(0.0f,-2.0f);
                        boxes[1].moveBy(-1.0f,-1.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(1.0f,1.0f);
                }
                else if (rotation == 180.0f)
                {
                        boxes[0].moveBy(2.0f,0.0f);
                        boxes[1].moveBy(1.0f,-1.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(-1.0f,1.0f);
                }
                else if (rotation == 270.0f)
                {
                        boxes[0].moveBy(0.0f,2.0f);
                        boxes[1].moveBy(1.0f,1.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(-1.0f,-1.0f);
                }
                else if (rotation == 0.0f)
                {
                        boxes[0].moveBy(-2.0f,0.0f);
                        boxes[1].moveBy(-1.0f,1.0f);
                        boxes[2].moveBy(0.0f,0.0f);
                        boxes[3].moveBy(1.0f,-1.0f);
                }
                break;                    
        }

        for (int i = 0; i < 4; ++i)
        {
                if (boxes[i].posX() < 0)
                {
                        boxes[0].moveBy(1.0f,0.0f);
                        boxes[1].moveBy(1.0f,0.0f);
                        boxes[2].moveBy(1.0f,0.0f);
                        boxes[3].moveBy(1.0f,0.0f);
                }
                else if (boxes[i].posX() >= FIELD_X)
                {
                        boxes[0].moveBy(-1.0f,0.0f);
                        boxes[1].moveBy(-1.0f,0.0f);
                        boxes[2].moveBy(-1.0f,0.0f);
                        boxes[3].moveBy(-1.0f,0.0f);        
                }
        }
        
                        
}
