/***************************************************************************
                          sfadelogo.cpp  -  description
                             -------------------
    begin                : Sun Dec 22 2002
    copyright            : (C) 2002 by Andre Stein
    email                : andre_stein@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sfadelogo.h"

/* CONSTS */
const float X_SIZE = 1.0f;
const float Y_SIZE = 0.5f;
const float MAX_SIZE = 3.0f;
const float SCALE_SPEED = 0.0005f;
const float NORMAL_SPEED = 0.0002f;

SFadeLogo::SFadeLogo()
{
        fadeType = NORMAL_FADE;
        fadeFactor = 0.0f;
        fadeSpeed = 0.0f;
}

SFadeLogo::~SFadeLogo()
{
}

/** No descriptions */
void SFadeLogo::draw()
{
        glPushMatrix();
        texture.select();
        glEnable(GL_BLEND);
        
        //fade with changing scale
        if (fadeType == SCALE_FADE)
        {
                /** scale depending on current size **/
                glScalef(fadeFactor, fadeFactor, 1.0f);
                                          
                glBegin(GL_QUADS);
                        glColor4f(1.0f,1.0f,1.0f, 1.0f-fadeFactor/MAX_SIZE);
                        glTexCoord2f(0.0f,0.0f); glVertex2f(-X_SIZE,-Y_SIZE);
                        glTexCoord2f(1.0f,0.0f); glVertex2f(X_SIZE,-Y_SIZE);
                        glTexCoord2f(1.0f,1.0f); glVertex2f(X_SIZE,Y_SIZE);
                        glTexCoord2f(0.0f,1.0f); glVertex2f(-X_SIZE,Y_SIZE);
                glEnd();
        }
        //normal fade
        else if (fadeType == NORMAL_FADE)
        {
                glBegin(GL_QUADS);
                        glColor4f(1.0f,1.0f,1.0f, fadeFactor);
                        glTexCoord2f(0.0f,0.0f); glVertex2f(-X_SIZE,-Y_SIZE);
                        glTexCoord2f(1.0f,0.0f); glVertex2f(X_SIZE,-Y_SIZE);
                        glTexCoord2f(1.0f,1.0f); glVertex2f(X_SIZE,Y_SIZE);
                        glTexCoord2f(0.0f,1.0f); glVertex2f(-X_SIZE,Y_SIZE);
                glEnd();                        
        }

        glDisable(GL_BLEND);
}

/** No descriptions */
void SFadeLogo::update(float frametime)
{
        //depending on fade
        if (fadeType == SCALE_FADE)
                fadeFactor += frametime * SCALE_SPEED;
        else if (fadeType == NORMAL_FADE)
                fadeFactor += frametime * fadeSpeed;

        //draw!
        draw();
}

/** No descriptions */
void SFadeLogo::init(const char* filename, SFadeType fade, float speed)
{
        texture.createMipmap(filename);
        fadeType = fade;

        if (speed == 0.0f)
                fadeSpeed = NORMAL_SPEED;
        else
                fadeSpeed = speed;

        //init values depending on fade
        if (fadeType == SCALE_FADE)
                fadeFactor = 1.0f;
        else if (fadeType == NORMAL_FADE)
                fadeFactor = 0.0f;
}

/** No descriptions */
bool SFadeLogo::isFinished()
{
        //depending on fade
        if (fadeType == SCALE_FADE)
                return ( fadeFactor >= MAX_SIZE );
        else if (fadeType == NORMAL_FADE)
                return ( fadeFactor >= 1.0f );

        return true;
}
