        /***************************************************************************
                          sgamefield.cpp  -  description
                             -------------------
    begin                : Sun Dec 22 2002
    copyright            : (C) 2002 by Andre Stein
    email                : andre_stein@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sgamefield.h"
#include <stdlib.h>
#include <time.h>
#include <math.h>

/**
  Helper function that has to be used
  as C++ doesn't allow class internal
  callback functions. At least this works hehe
 **/ 
Uint32 timerCallback(Uint32 interval, void* param)
{
        //Call the real funtion
        ((SGamefield*)param)->timerEvent();

        //continue timer
        return interval;
}


bool SGamefield::field[FIELD_X][FIELD_Y];

SGamefield::SGamefield()
{
        //int rand()
        srand(time(NULL));

        reset();
        configFile = 0;
        timerID = 0;
}

SGamefield::~SGamefield()
{
}

/** No descriptions */
void SGamefield::reset()
{
        for (int i = 0; i < FIELD_X; ++i)
        {
                for (int j = 0; j < FIELD_Y; ++j)
                {
                        field[i][j] = false;
                }
        }

        //score
        score = 0;
        level = 1;
        removedLines = 0;
        cantMove = false;
        nextBrick = SBrickType(rand() % BRICK_NUM);
        paused = false;
        //clear if there are any left
        boxes.clear();       
}

/** No descriptions */
void SGamefield::init(SConfiguration* config)
{
        configFile = config;

        /** Textures **/
        textures[0].createTexture("data/box_white.bmp");
        textures[1].createTexture("data/box_white2.bmp");
        textures[2].createTexture("data/box_white3.bmp");
        textures[3].createTexture("data/box_red.bmp");
        textures[4].createTexture("data/box_red2.bmp");
        textures[5].createTexture("data/box_orange.bmp");
        textures[6].createTexture("data/box_orange2.bmp");

        /**
          Create gamefield sides object
         **/

        GLUquadric* quadric = gluNewQuadric();
	gluQuadricNormals(quadric,GLU_SMOOTH);
	gluQuadricTexture(quadric,GL_FALSE);

        sidesObject = glGenLists(1);

        glNewList(sidesObject, GL_COMPILE);
                glColor4f(1.0f,1.0f,1.0f,0.5f);
                /* LEFT */
        	glTranslatef(-2*CYLINDER_RADIUS,FIELD_Y,0.0f);
        	glRotatef(90.0f,1.0f,0.0f,0.0f);

	        gluCylinder(quadric,CYLINDER_RADIUS,CYLINDER_RADIUS,
			FIELD_Y,32,1);
        	/* RIGHT */
         	glRotatef(-90.0f,1.0f,0.0f,0.0f);
                glTranslatef(2*CYLINDER_RADIUS,-FIELD_Y,0.0f);
	  
	        glTranslatef(FIELD_X,FIELD_Y,0.0f);
        	glRotatef(90.0f,1.0f,0.0f,0.0f);

        	gluCylinder(quadric,CYLINDER_RADIUS,CYLINDER_RADIUS,
			FIELD_Y,32,1);
	        /* BOTTOM */
                glRotatef(-90.0f,1.0f,0.0f,0.0f);
                glTranslatef(-FIELD_X,-FIELD_Y,0.0f);
	
                glTranslatef(-2*CYLINDER_RADIUS,-CYLINDER_RADIUS,0.0f);
	        glRotatef(90.0f,0.0f,1.0f,0.0f);

	        gluCylinder(quadric,CYLINDER_RADIUS,CYLINDER_RADIUS,
			FIELD_X + 2* CYLINDER_RADIUS,32,1);
                glRotatef(-90.0f,1.0f,0.0f,0.0f);
                glTranslatef(2*CYLINDER_RADIUS,CYLINDER_RADIUS,0.0f);
        glEndList();

        //cleanup
        gluDeleteQuadric(quadric);

        /** Mirror object */
        
        mirrorObject = glGenLists(1);

        glNewList(mirrorObject, GL_COMPILE);
                glBegin(GL_QUADS);
                        glColor4f(0.5f,0.5f,0.5f,0.75f);
                        glVertex2f(-1.0f,-1.0f);
                        glVertex2f(FIELD_X+1.0f,-1.0f);
                        glVertex2f(FIELD_X+1.0f,FIELD_Y+1.0f);
                        glVertex2f(-1.0f, FIELD_Y+1.0f);
                glEnd();
        glEndList();
}

/** No descriptions */
void SGamefield::drawField()
{
        //draw sides of gamefield
        glEnable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);

        glPushMatrix();
        glCallList(sidesObject);
        glPopMatrix();

	glDisable(GL_BLEND);
        glEnable(GL_TEXTURE_2D);

        /**
         * BOXES
         ***********/

        //box array

        for (unsigned int i = 0; i < boxes.size(); ++i)
        {
                boxes[i].update(frametime);
                //boxes[i].draw();
        }

        current.draw();
}

/** No descriptions */
void SGamefield::draw()
{
        //Draw field and depending on value in
        //configuration field with reflection

        glPushMatrix();
        
        /** DRAW REFLECTION **/
        glTranslatef(-8.0f,-11.0f,-50.0f);

        if (configFile->isReflection())
                drawField();

        /** DRAW MIRROR **/
        glTranslatef(0.0f,0.0f,10.0f);

        if (configFile->isReflection())
        {
                glEnable(GL_BLEND);
                glDisable(GL_TEXTURE_2D);
        
                glCallList(mirrorObject);
        
                glDisable(GL_BLEND);
                glEnable(GL_TEXTURE_2D);
        }

        /** DRAW REAL FIELD **/

        glTranslatef(-2.0f,0.0f,10.0f);

        drawField();
                        
        glPopMatrix();

}

/** No descriptions */
void SGamefield::update(float timeOfFrame)
{
        frametime = timeOfFrame;

        //Everything will be done by timer so we
        //don't have to worry..
        
        /** Draw **/
        draw();
}

/** No descriptions */
void SGamefield::moveLeft()
{
        current.moveLeft();
}

/** No descriptions */
void SGamefield::moveRight()
{
        current.moveRight();
}

/** No descriptions */
void SGamefield::moveDown()
{
        //go down faster much faster
        SDL_RemoveTimer(timerID);
        timerID = SDL_AddTimer(FAST_REACTION_TIME, timerCallback, (void*)this);      
}           

/** No descriptions */
void SGamefield::changeForm()
{
        current.changeForm();
}

/** No descriptions */
void SGamefield::newBrick()
{
        current.create(nextBrick, textures);
        current.moveBy(5.0f,FIELD_Y);
        //reaction gets more little depending
        //on level
        //current.setSpeed(DEFAULT_SPEED*level);
        //current.setState(FALLING);
        //immediately calculate next brick
        nextBrick = SBrickType(rand() % BRICK_NUM);

        /** Setup timer depeding on level **/
        SDL_RemoveTimer(timerID);
        timerID = SDL_AddTimer(REACTION_TIME - (level*PERLEVEL_TIME_DECREASE), timerCallback, (void*)this);
}

/** No descriptions */
void SGamefield::finishRound()
{
        //reset
        cantMove = false;
        //current.setState(NORMAL);

        /** add brick to box vector **/

        boxes.push_back(current.getBoxes()[0]);
        boxes.push_back(current.getBoxes()[1]);
        boxes.push_back(current.getBoxes()[2]);
        boxes.push_back(current.getBoxes()[3]);
        
        /** update field array **/

        updateField();

        /** check for rows destroyed **/

        destroyRows();
        
        // new brick!
        newBrick();
}

/** No descriptions */
bool SGamefield::isGameOver()
{
        //when brick reaches the top in the middle of the screen
        // (3.0f) it's time to stop
        for (unsigned int i = 0; i < boxes.size(); ++i)
        {
                if (boxes[i].posX() == 5.0f && boxes[i].posY() == FIELD_Y)
                {
                        //destroy timer
                        SDL_RemoveTimer(timerID);
                                                
                        return true;
                }
        }

        return false;
}

/** No descriptions */
void SGamefield::destroyRows()
{
        int lines = 0;
        
        for (int y = 0; y < FIELD_Y; ++y)
        {
                if (field[0][y] && field[1][y] && field[2][y] && field[3][y] &&
                    field[4][y] && field[5][y] && field[6][y] && field[7][y] &&
                    field[8][y] && field[9][y] && field[10][y])
                {
                        
                        for (unsigned int i = 0; i < boxes.size(); ++i)
                        {
                                //if it's a box from this row, erase
                                if (boxes[i].posY() == y)
                                {
                                        //erase box and decrement counter
                                        boxes.erase(&boxes[i]);
                                        i--;      
                                }
                                //if it's a box above, move down
                                else if (boxes[i].posY() > y)
                                {
                                        boxes[i].moveBy(0.0f,-1.0f);
                                }                                 
                        }
         
                        //an other line
                        lines++;
                        //as we deleted one row, we go back and
                        //update all fields
                        --y;
                        updateField();
                }
                
                
        }

        //calculate points
        score += (int)pow( level * 5, lines);
        removedLines += lines;
        //updateField();       
}

/** No descriptions */
void SGamefield::updateField()
{
        for (int i = 0; i < FIELD_X; ++i)
        {                        
                for (int j = 0; j < FIELD_Y; ++j)
                {
                        field[i][j] = false;
                }
        }

        for (unsigned int box = 0; box < boxes.size(); ++box)
        {
                field[boxes[box].posX()][boxes[box].posY()] = true;
        }
}

/** No descriptions */
void SGamefield::timerEvent()
{
        current.moveDown();
        
        //on every level, the per reaction time score gets higher
        score += level;

        if (current.touchesBottom() || current.touchesBoxes(BOTTOM) )
        {
                if ( cantMove )
                        finishRound();
                else
                        cantMove = true;
        }

        //increment level every 10 removed lines
        if (removedLines != 0)
        {
                level = int(removedLines/10)+1;
        }       
}

/** No descriptions */
void SGamefield::pause()
{
        paused = !paused;

        if (paused)
        {
                SDL_RemoveTimer(timerID);
        }
        else
        {
                timerID = SDL_AddTimer(REACTION_TIME - (level*PERLEVEL_TIME_DECREASE), timerCallback, (void*)this);                       
        }
}
